#include <errno.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
# include <sys/un.h>

#ifndef offsetof
# define offsetof(sname,fname) ((long)&((sname *)0)->fname)
#endif
#ifndef max
# define max(a,b) ((a) > (b) ? (a) : (b))
#endif
#define SERVPORT 2220

static void display_inet(struct sockaddr_in *s, struct sockaddr_in *d)
{
    char *s1,*d1;

    if((s1 = strdup(inet_ntoa(s->sin_addr))))
    {
	if((d1 = strdup(inet_ntoa(d->sin_addr))))
	{
	    fprintf(stderr, "s = %s.%d <--> p = %s.%d (%x %x)\n", 
		    s1, htons(s->sin_port),
		    d1, htons(d->sin_port),
		    s->sin_addr.s_addr, d->sin_addr.s_addr);
	    free(d1);
	}
	free(s1);
    }
}

void usage(void)
{
    fputs ("Usage: tc [-p port  -h host]|[-u path]\n", stderr);
    exit (EXIT_FAILURE);
}

int  getsocket(struct sockaddr *iname, size_t *nlen, int port,
               char *host, char * fn)
{
    struct hostent *h;
    int sock,domain;
           
    if(fn == NULL)
    {
        if(host == NULL)
        {
            host = malloc(256);
            if(gethostname (host, 256))
            {
                perror ("gethostname");
                exit(0);
            }
        }

        if ((h = (struct hostent *) gethostbyname (host)) == NULL)
        {
            perror ("gethostbyname");
            exit(0);
        }

        ((struct sockaddr_in *)iname)->sin_family = h->h_addrtype;
        ((struct sockaddr_in *)iname)->sin_port = htons (port);
        ((struct sockaddr_in *)iname)->sin_addr =
            *((struct in_addr *) h->h_addr);
        *nlen = sizeof(struct sockaddr_in);
        domain = AF_INET;
    }
    else
    {
        ((struct sockaddr_un *)iname)->sun_family = AF_UNIX;
        strcpy (((struct sockaddr_un *)iname)->sun_path, fn);
        *nlen = SUN_LEN((struct sockaddr_un *)iname);
        domain = AF_UNIX;      
    }

    if ((sock = socket (domain, SOCK_STREAM, 0)) == -1)
    {
	perror ("socket");
	exit (EXIT_FAILURE);
    }
    return sock;
}
	    
int main (int ac, char **av)
{
    struct sockaddr *name;
    int sock, n, retval;
    char *host = NULL;
    int c, port=SERVPORT;
    short flag = 0;
    char *fn = NULL;
    int nlen;

    nlen = max(sizeof(struct sockaddr_in), sizeof(struct sockaddr_un));    
    name = malloc(nlen);
    
    if (ac < 2)
    {
	usage();
    }

    flag = 0;
    while((c = getopt(ac, av, "p:h:u:")) != EOF)
    {
	switch(c)
	{
            case 'p':
                port = atoi(optarg);
                break;
            case 'h':
                host = strdup(optarg);
                break;
            case 'u':
                fn = optarg;
                break;
            case '?':
                usage ();
                break;
	}	    
    }

    if(!host && !fn) exit(0);

    sock = getsocket (name, (size_t*)&nlen, port, host, fn);
    

    retval = connect (sock, name, nlen);

    if (retval)
    {
	perror ("connect");
    }
    else
    {
	char buf[1024];
	while(fgets(buf, 1024, stdin))
	{
	    n = strlen(buf);
	    retval = send (sock, buf, n, 0);
	    if(retval < 0)
	    {
		perror("send");
	    }
	    else
	    {
		if((n = recv(sock, buf, sizeof(buf), 0)) > 0)
		{
		    fputs(buf, stdout);
		}
		else
                {
                    perror("recv");
                    break;
                }
	    }
	}
    }
    return EXIT_SUCCESS;
}


